#include "stdafx.h"
#include "AnimatedSprite.h"
#include "TextureManager.h"

namespace LD46
{
	AnimatedSprite::AnimatedSprite(TextureManager texMgr, const int resourceId, const int width, const int height, const float speed, bool playOnce, const int originX/* = -1*/, const int originY/* = -1*/)
		:m_texMgr(texMgr)
		, m_width(width)
		, m_height(height)
		, m_speed(speed)
		, m_currentFrame(0.0f)
		, m_revesePlay(false)
		, m_play(false)
		,m_playOnce(playOnce)
		, m_isFinished(true)
	{
		m_texture = m_texMgr.GetTexture(resourceId);			
		m_texture.setSmooth(false);
		auto w = originX != -1 ? originX : width / 2;
		auto h = originY != -1 ? originY : height / 2;
		m_sprite1.setOrigin(w, h);
		m_sprite2.setOrigin(w, h);
		m_sprite1.setTexture(m_texture);
		m_sprite2.setTexture(m_texture);
		m_totalFrames = m_texture.getSize().y / height;
		UpdateSpriteMapping();
		m_firstRound = true;
	}

	AnimatedSprite::~AnimatedSprite()
	{
	}

	bool AnimatedSprite::Update(const sf::Time& tm)
	{
		if (m_firstRound)
		{
			m_prevTime = tm;
			m_firstRound = false;
		}

		if (m_play)
		{
			auto elapsed = m_speed * static_cast<float>((tm - m_prevTime).asMilliseconds()) / 100.0f;

			if (m_revesePlay)
				Previous(elapsed);
			else
				Next(elapsed);
		}
		m_prevTime = tm;
		return true;
	}
	
	void AnimatedSprite::Draw(sf::RenderWindow& wnd, sf::Shader* shdr)
	{
		if (shdr != nullptr)
		{
			sf::RenderStates states;
			states.shader = shdr;
			states.texture = &m_texture;
			wnd.draw(m_sprite1, states);
			wnd.draw(m_sprite2, states);
		}
		else
		{
			wnd.draw(m_sprite1);
			wnd.draw(m_sprite2);
		}
	}

	void AnimatedSprite::Play(bool reverse)
	{
		m_isFinished = false;
		m_play = true;
		m_revesePlay = reverse;
	}

	void AnimatedSprite::Stop()
	{
		m_isFinished = true;
		m_currentFrame = 0.0;
		m_play = false;
	}

	void AnimatedSprite::Next(const float elapsedTime)
	{
		m_currentFrame += elapsedTime;
		if (m_currentFrame >= m_totalFrames)
		{
			m_currentFrame = 0;
			if (m_playOnce)
			{
				Stop();
				return;
			}
		}

		UpdateSpriteMapping();
	}

	void AnimatedSprite::Previous(const float elapsedTime)
	{
		m_currentFrame -= elapsedTime;
		if (m_currentFrame < 0)
		{
			m_currentFrame = m_totalFrames - 1;
			if (m_playOnce)
			{
				Stop();
				return;
			}
		}

		UpdateSpriteMapping();
	}
	sf::Vector2f AnimatedSprite::Pos() const
	{
		return m_sprite1.getPosition();
	}
	void AnimatedSprite::SetPos(const sf::Vector2f& pos)
	{
		m_sprite1.setPosition(pos);
		m_sprite2.setPosition(pos);
	}
	void AnimatedSprite::SetScale(const sf::Vector2f& scale)
	{
		m_sprite1.setScale(scale);
		m_sprite2.setScale(scale);
	}

	const sf::Vector2f& AnimatedSprite::Scale() const
	{
		return m_sprite1.getScale();
	}
	
	void AnimatedSprite::SetOrigin(const sf::Vector2f& center)
	{
		m_sprite1.setOrigin(center);
		m_sprite2.setOrigin(center);
	}

	bool AnimatedSprite::IsFinished() const
	{
		return m_isFinished;
	}

	float AnimatedSprite::CurrentFrame() const
	{
		return m_currentFrame;
	}

	void AnimatedSprite::UpdateSpriteMapping()
	{
		auto cl = ceil(m_currentFrame);
		if (cl == m_currentFrame)
			cl += 1.0f;

		auto flr = floor(m_currentFrame);
		auto diff = cl - m_currentFrame;
		if (cl >= m_totalFrames)
			cl = 0.0;

		m_sprite1.setTextureRect(sf::IntRect(0, m_height * flr, m_width, m_height));
		m_sprite2.setTextureRect(sf::IntRect(0, m_height * cl, m_width, m_height));
		//m_sprite1.setColor(sf::Color(255, 255, 255, 255 * diff));
		m_sprite1.setColor(sf::Color(255, 255, 255, 255));
		m_sprite2.setColor(sf::Color(255, 255, 255, 255 * (1.0f - diff)));
	}
}