#include "stdafx.h"
#include "EnemyMgr.h"
#include "Hero.h"
#include "TextureManager.h"
#include "SoundManager.h"
#include "ParticleMgr.h"
#include "resource.h"
#include "Hud.h"
#include "Utils.h"
#include "Level.h"

namespace LD46
{
	EnemyMgr::EnemyMgr(TextureManager& txMgr, SoundManager& soundMgr, ParticleMgr& partMgr, Hud& hud, const int level)
		:m_txMgr(txMgr)
		, m_sndMgr(soundMgr)
		, m_partMgr(partMgr)
		,m_hud(hud)
		, m_level(level)
		, m_prevColl(CollisionType::None)
		,m_firstRound(true)
		, m_metMystic()
	{
		for (int i = 0; i != 4; ++i)
			m_bossSpawned[i] = false;

		m_spawnLimits[Enemy::EnemyType::Bat] = 1000;
		m_spawnLimits[Enemy::EnemyType::Blob] = 2000;
		m_spawnLimits[Enemy::EnemyType::Gargoyle] = 5000;
		m_spawnLimits[Enemy::EnemyType::Demon] = 5000;
		m_spawnLimits[Enemy::EnemyType::Bug] = 500;

		//auto nme = std::make_shared<Enemy>(txMgr, Enemy::EnemyType::Bug, 10.0);
		//nme->SetPosition(sf::Vector3f(129.0f, 3.0f, 8000.0f));
		//m_enemies[nme->Id()] = nme;

		auto nme = std::make_shared<Enemy>(txMgr, Enemy::EnemyType::Mystic, 50.0);
		nme->SetPosition(sf::Vector3f(129.0f, 16.0f, 7900.0f));
		m_enemies[nme->Id()] = nme;
	}

	EnemyMgr::~EnemyMgr()
	{
	}

	void EnemyMgr::Update(const sf::Time& tm, Hero &hero, Level& level)
	{
		const auto& requests = level.SpawnRequest();
		for (auto& r : requests)
		{
			auto dx = hero.Position().x - r.second.x;
			auto dz = hero.Position().y - r.second.z;
			auto dd = sqrtf(dx * dx + dz * dz);
			if (dd > 80)
				continue;

			auto et = static_cast<Enemy::EnemyType>(r.first);
			switch (et)
			{
			case Enemy::EnemyType::Boss1:
			case Enemy::EnemyType::Boss2:
			case Enemy::EnemyType::Boss3:
			case Enemy::EnemyType::Boss4:
				//m_bossSpawned
				if (!m_bossSpawned[et - Enemy::EnemyType::Boss1])
				{
					//level.SetMap(r.second.x / 32, r.second.y / 32, 0);

					m_bossSpawned[et - Enemy::EnemyType::Boss1] = true;
					auto nme = std::make_shared<Enemy>(m_txMgr, et, 10.0, this);
					nme->SetPosition(r.second);
					m_enemies[nme->Id()] = nme;
				}
				break;
			default:
				auto& lastSpawnTime = m_lastSpawnTime[et];
				if ((tm - lastSpawnTime).asMilliseconds() > m_spawnLimits[et])
				{
					lastSpawnTime = tm;
					auto nme = std::make_shared<Enemy>(m_txMgr, et, 10.0);
					nme->SetPosition(r.second);
					m_enemies[nme->Id()] = nme;
				}
				break;
			}
		}
		level.ClearSpawnRequest();
		UpdateEnemies(tm, hero, level);
	}

	bool EnemyMgr::UpdateShotCollision(const sf::Vector3f& shotPos)
	{
		for (auto& enemyPair : m_enemies)
		{
			if (enemyPair.second->IsCollision(shotPos))
			{
				//Only collide with one object
				enemyPair.second->AdjustPower(-0.1);
			}
		}
		return false;
	}

	Enemy* EnemyMgr::CreateTentacle(sf::Vector3f& pos)
	{
		auto nme = std::make_shared<Enemy>(m_txMgr, Enemy::EnemyType::Tentacle, 5.0);
		nme->SetPosition(pos);
		m_enemies[nme->Id()] = nme;
		return nme.get();
	}

	Enemy* EnemyMgr::CreateEnemy(const sf::Vector3f& pos, Enemy::EnemyType et)
	{
		auto nme = std::make_shared<Enemy>(m_txMgr, et, et == Enemy::EnemyType::Tusk ? 20.0 : 10.0);
		nme->SetPosition(pos);
		m_enemies[nme->Id()] = nme;
		return nme.get();
	}

	void EnemyMgr::Erase(int id)
	{
		m_enemies.erase(id);
	}

	std::optional<sf::Time> EnemyMgr::MetMystic() const
	{
		return m_metMystic;
	}

	void EnemyMgr::UpdateEnemies(const sf::Time& tm, LD46::Hero& hero, LD46::Level& level)
	{
		std::vector<long> deadEnemies;
		for (auto& enemyPair : m_enemies)
		{
			if (!enemyPair.second->Update(tm, hero, level))
			{
				deadEnemies.push_back(enemyPair.first);
				switch (enemyPair.second->TheType())
				{
				case Enemy::EnemyType::Boss1:
				case Enemy::EnemyType::Boss2:
				case Enemy::EnemyType::Boss3:
				case Enemy::EnemyType::Boss4:
					CreateEnemy(enemyPair.second->Position(), Enemy::EnemyType::Tusk);
					break;
				default:
					break;
				}
				if (enemyPair.second->TheType() == Enemy::EnemyType::Boss1)
				{
					for (auto& e : enemyPair.second->Tentacles())
					{
						deadEnemies.push_back(e->Id());
					}
				}
			}
			else
			{
				auto hp = level.HeroPos();
				auto& ep = enemyPair.second->Position();
				auto d = hp - ep;
				auto l = sqrtf(d.x * d.x + d.z * d.z);
				if (l > (enemyPair.second->Radius()*0.5))
					continue;

				if (hero.PerformingHit())
				{
					auto pId = m_partMgr.CreateParticle(ep, ParticleMgr::ParticleType::HitEnemy, 200);
					m_partMgr.SetParticlePos(pId, sf::Vector3f(ep.x, ep.y, ep.z));
					level.SetParticle(0.5*ep.x, 0.5*ep.z, pId);
					enemyPair.second->AdjustPower(-0.5);
				}
				else
				{
					if (enemyPair.second->TheType() == Enemy::EnemyType::Mystic)
					{
						m_metMystic = tm;
						auto r = hero.Rotation();
						hero.SetKnockBack(sf::Vector2f(sin(r) * 3, cos(r) * 3));
						m_hud.TextDisplay("We share a bond you and I..\nKeeping me alive will keep you alive.\nPlease find the pieces of my missing tusk.\nI've given you a splinter of my tusk to help you on this quest.\n", 20000);
					}
					else if (enemyPair.second->TheType() == Enemy::EnemyType::Tusk)
					{
						hero.AddTusk();
						if (hero.Tusks() == 3)
						{
							m_hud.TextDisplay("Congratulations!You found all the tusk pieces!\nYou have the patience and persistens\nof a master adventurer!\n", 50000);
						}
						else
							m_hud.TextDisplay("A piece of the tusk only " + std::to_string(3 - hero.Tusks()) + " left to go!",2000);
						deadEnemies.push_back(enemyPair.first);
					}
					else
					{
						if ((tm - m_lastHeroHit).asMilliseconds() > 1000)
						{
							auto hp = hero.Position();
							auto pId = m_partMgr.CreateParticle(ep, ParticleMgr::ParticleType::HitHero, 300);
							m_partMgr.SetParticlePos(pId, sf::Vector3f(hp.x, 5, hp.y));
							level.SetParticle(0.5 * hp.x, 0.5 * hp.y, pId);
							hero.AdjustPower(-0.1, hero.GetWorldState());
							auto r = hero.Rotation();
							hero.SetKnockBack(sf::Vector2f(sin(r) * 10, cos(r) * 10));
							m_lastHeroHit = tm;
						}
					}
				}
			}
		}
		for (auto id : deadEnemies)
		{
			auto iter = m_enemies.find(id);
			if (iter == std::end(m_enemies))
				continue;

			auto ep = iter->second->Position();
			level.ClearEnemy(ep);
			auto pt = ParticleMgr::ParticleType::Banish;
			switch (iter->second->TheType())
			{
			case Enemy::EnemyType::Tusk:
				pt = ParticleMgr::ParticleType::HeroShot;
				break;
			default:
				break;
			}
			auto pId = m_partMgr.CreateParticle(ep, pt, 350);
			m_partMgr.SetParticlePos(pId, sf::Vector3f(ep.x, ep.y+10, ep.z));
			level.SetParticle(0.5 * ep.x, 0.5 * ep.z, pId);
			m_enemies.erase(id);
		}

	}
}